;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10; Package: photo -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          dof.lisp
;;;; Purpose:       Depth of field functions for cl-photo
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  April 2005
;;;;
;;;; $Id$
;;;;
;;;; This file, part of cl-photo, is Copyright (c) 2005 by Kevin Rosenberg.
;;;; Rights of modification and redistribution are in the LICENSE file.
;;;;
;;;; *************************************************************************

(in-package #:photo)

;; Based on http://www.photostuff.co.uk/dofmstr.htm

(defun coc-format (format &key (lpm 5) (minimum-distance 250) 
                          (viewing-distance 250)
                          (print-size (format-dimensions :8x10in)))
  "Returns circle of confusion in mm and print magnification for a format. 
Default resolving power is 5 lpm at 25cm."

  (let* ((format-size (format-dimensions format))
         (format-diagonal (diagonal (car format-size) (cdr format-size)))
         (print-diagonal (diagonal (car print-size) (cdr print-size)))
         (resolution-factor (/ (* lpm print-diagonal minimum-distance)
                               (* format-diagonal viewing-distance)))
         (coc (/ 1.0d0 resolution-factor))
         (print-magnification (/ print-diagonal format-diagonal)))
    (values coc print-magnification)))

(defun coc-sensor (format nx ny)
  "Returns circle of confusion based on pixel size."
  (let* ((dim (format-dimensions format))
         (pixel-width (/ (car dim) nx))
         (pixel-height (/ (cdr dim) ny))) 
    (values (coerce (* 2 pixel-width) 'float)
            (coerce (* 2 pixel-height) 'float))))

(defun coc-sensor-camera (camera &key (format :aps))
  (let ((dim (sensor-dimensions camera :format format)))
    (coc-sensor format (car dim) (cdr dim))))

(defun sensor-dimensions-megapixels (format megapixels)
  (let* ((dim (format-dimensions format))
         (aspect-ratio (/ (car dim) (cdr dim)))
         (width (round (sqrt (* aspect-ratio 1000000 megapixels)))))
    (cons width (round (/ width aspect-ratio)))))

(defun sensor-dimensions (camera &key (format :aps))
  (etypecase camera
    (keyword
     (ecase camera
       ;; nikon
       (:d2x (cons 4288 2848))
       (:d2x (cons 2484 1242))
       (:d70 (cons 3008 2000))

       ;; canon
       (:1d (cons 2464 1648))
       (:1d2 (cons 3504 2336))
       (:1ds (cons 4064 2704))
       (:1ds2 (cons 4992 3328))

       ))
    (number
     (sensor-dimensions-megapixels format camera))))

(defun coc-airy-disk (f-stop)
  "Return the circle of confusion based on the airy disk."
  (let ((airy (/ f-stop 1500)))
    (coerce (* 2 airy) 'float)))

(defun dof (focal-length f-stop distance coc)
  "Returns depth of field as fives values: 
near dof, far dof, total dof, near point, far point.
Circle of confusion can either be a number or keyword designating format."
  (let* ((aperature (/ focal-length f-stop))
         (numerator (* distance coc (- distance focal-length)))
         (factor-1 (* focal-length aperature))
         (factor-2 (* coc (- distance focal-length)))
         (near (/ numerator (+ factor-1 factor-2)))
         (far (/ numerator (- factor-1 factor-2)))
         (depth (+ far near)))
    (values near far depth (- distance near) (+ distance far))))

(defun dof-feet (focal-length f-stop distance coc)
  (multiple-value-bind (near-dof far-dof total-dof near-point far-point)
      (dof focal-length f-stop (feet->mm distance) coc)
    (values (mm->feet near-dof) (mm->feet far-dof) (mm->feet total-dof)
            (mm->feet near-point) (mm->feet far-point))))

(defun dof-meters (focal-length f-stop distance coc)
  (multiple-value-bind (near-dof far-dof total-dof near-point far-point)
      (dof focal-length f-stop (* 1000 distance) coc)
    (values (* 0.001 near-dof) (* 0.001 far-dof) (* 0.001 total-dof)
            (* 0.001 near-point) (* 0.001 far-point))))

(defun hyperfocal (focal-length f-stop coc)
  (+ focal-length (/ (* focal-length focal-length) (* f-stop coc))))

(defun hyperfocal-feet (focal-length f-stop coc)
  (mm->feet (hyperfocal focal-length f-stop coc)))


